<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @subpackage import
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: template.php 208 2008-08-31 08:36:07Z lks $
 */

header('Content-Type: text/html; charset=utf-8');
chdir('../../');

if (strtolower($_SERVER['REQUEST_METHOD']) != 'post') {
    ?>
<form method="post">
<input type="submit" value="Начать импорт" />
</form>
    <?php
    exit;
}

/** zengine */
require_once 'zengine/classes/zengine.php';

zengine::init();

$config = new zconfig('import_cmsimple_template.ini');
$cmsimple = new zimport_cmsimple_template();
$cmsimple->setCharsetFrom($config->get('charset.from', 'windows-1251'));
$cmsimple->setCharsetTo($config->get('charset.to', 'utf-8'));

/**
 * Находим все папки
 */
$directory = new DirectoryIterator(dirname(__FILE__));
foreach ($directory as $file) {
    if ($file->isDir() && !$file->isDot()) {
        if (file_exists($file->getPathname() . DIRECTORY_SEPARATOR . 'template.htm')) {
            $cmsimple->convert($file->getPathname());
        }
    }
}

$logs = array();
$logs = $cmsimple->getLog();
$logs[] = '------------';
$logs[] = '<a href="' . zengine::getInstance()->getBaseUri() . '">Открыть сайт</a>';

echo '<pre>' . implode("\r\n", $logs) . '</pre>';

class zimport_cmsimple_template {
    /**
     * Устанавливает кодировку исходного шаблона
     *
     * @param string $charset кодировка
     */
    public function setCharsetFrom ($charset) {
    	$this->_fromCharset = $charset;
    }

    /**
     * Устанавливает кодировку конечного шаблона
     *
     * @param string $charset кодировка
     */
    public function setCharsetTo ($charset) {
        $this->_toCharset = $charset;
    }

    /**
     * Копирует папку шаблона и производит конвертацию шаблона из CMSimple в Zebrum Lite
     *
     * @param string $templateDir путь к папке с шаблоном
     */
    public function convert ($templateDir) {
    	$templateName = basename($templateDir);
        if (is_dir('templates/' . $templateName)) {
            $this->log('Шаблон "' . $templateName . '" уже установлен в системе. Импорт отменен.');
            return;
        }

        if (!rename($templateDir, 'templates/' . $templateName)) {
            $this->log('Не удалось переместить шаблон "' . $templateName . '" в папку шаблонов системы');
            return;
        }

        rename('templates/' . $templateName . '/template.htm', 'templates/' . $templateName . '/template.php');
        $this->convertTemplate('templates/' . $templateName . '/template.php');
        $this->log('Конвертирован шаблон: ' . $templateName);
    }

    /**
     * Производит необходимые замены в файле шаблона
     *
     * @param string $filename путь к файлу шаблона
     */
    private function convertTemplate ($filename) {
        $template = file_get_contents($filename);

        $preg_replaces = array(
            // Возможные варианты заменяем на аналоги
            '%<\?(php echo|=)\s*?\$pth\[\'folder\'\]\[\'templateimages\'\];?\s*?\?>%U' => '~ztpl/images/',
            '%<\?(php echo|=)\s*?toc\(\);?\s*?\?>%U' => '<?php cmsimple_menu(); ?>',
            '%<\?(php echo|=)\s*?locator\(\);?\s*?\?>%U' => '<?php z_breadcrumbs(); ?>',
            '%<\?(php echo|=)\s*?sitemaplink\(\);?\s*?\?>%U' => '<?php z_sitemap(); ?>',
            '%<\?(php echo|=)\s*?submenu\(\);?\s*?\?>%U' => '<?php z_subpages(); ?>',
            '%<\?(php echo|=)\s*?content\(\);?\s*?\?>%U' => '<h1><?php z_h1(); ?></h1>
        <?php echo $this->zpage->getBody() ?>',
            '%<\?(php echo|=)\s*?head\(\);?\s*?\?>%U' => '<?php z_head(); ?>
        <link rel="stylesheet" type="text/css" href="~ztpl/stylesheet.css" />',
            // Все то, что мы знаем, но нету аналога, заменяем на пустоту
            '%<\?(php echo|=)\s*?(searchbox|editmenu|onload|previouspage|top|nextpage|login|lastupdate|sitename|loginlink|printlink|mailformlink|guestbooklink|languagemenu)\(\);?\s*?\?>%U' => '',
            '%<\?php\s+\/\*\s+CMSimple.*\?>\r?\n%smU' => '',
            '%Powered by CMSimple%iU' => 'Powered by Zebrum Lite',
            '%(www\.)?cmsimple\.[a-z]+?%iU' => 'blog.zebrum.ru',
            // Все, что не смогли определить, заменяем на текст, чтобы не было ошибок
            // сюда же попадут все недобросовестные инклуды
            '%<\?=(.+)\?>%U' => '$1',
            '%<\?(php)?\s*(include.+)\?>%U' => '$1',
        );

        foreach ($preg_replaces as $pattern => $to) {
            $template = preg_replace($pattern, $to, $template);
        }

        if ($this->_fromCharset != $this->_toCharset) {
            $template = iconv($this->_fromCharset, $this->_toCharset, $template);
        }

        zutils::file_put_contents($filename, $template);
    }

    /**
     * Возвращает массив с лог сообщениями
     *
     * @var array
     */
    public function getLog () {
        return $this->logs;
    }

    /**
     * Записывает сообщение в лог
     *
     * @param string $message
     */
    private function log ($message) {
        $this->logs[] = $message;
    }

    /**
     * Лог
     *
     * @var array
     */
    private $logs = array();

    /**
     * Кодировка исходного шаблона
     *
     * @var string
     */
    private $_toCharset = 'utf-8';

    /**
     * Кодировка конечного шаблона
     *
     * @var string
     */
    private $_fromCharset = 'utf-8';
}