<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zparser.php 495 2008-11-12 14:38:09Z lks $
 */

/** zsource_txt */
require_once 'zengine/classes/zsource/txt.php';

/**
 * Класс для разбивки большого текста на страницы
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zparser
{
    /**
     * Устанавливает кодировку входного файла по умолчанию
     *
     * @param string $charset кодировка, используемая по умолчанию
     */
    public function setDefaultCharset ($charset) {
    	if (!empty($charset)) {
    	    $this->charset = $charset;
    	}
    }

    /**
     * Устанавливает фильтр по умолчанию
     *
     * @param string $charset фильтр
     */
    public function setDefaultFilter ($filter) {
        if (!empty($filter)) {
            $this->filter = $filter;
        }
    }

    /**
     * Удалять старый контент
     *
     * @param boolean $deleteOld
     */
    public function deleteOldContent ($deleteOld) {
        $this->deleteOld = (bool) $deleteOld;
    }

    /**
     * Устанавливает флаг принудительного создания главной страницы
     *
     * @param boolean $force
     */
    public function forceIndex ($force) {
    	$this->forceIndex = (bool) $force;
    }

    /**
     * Метод разбивает файл на страницы и сохраняет их
     *
     * @return boolean
     */
    public function parse ($filename)
    {
        $this->init();
        $this->position = 1;
        $f = fopen($filename, 'r', false);
        if (!$f) {
            return false;
        }
        while (!feof($f)) {
            $this->convert(fread($f, 16*1024), feof($f));
        }
        fclose($f);

        if ($this->deleteOld) {
            /**
             * Удаляем весь лишний контент
             */
            foreach ($this->oldPages as $pagepath) {
                if (zengine::zsource()->delete($pagepath)) {
                    $this->log('Удалена старая страница: ' . $pagepath);
                } else {
                    $this->log('Не удалось удалить старую страницу: ' . $pagepath);
                }
            }
        }

        $this->log('Парсинг закончен.');
        return true;
    }

    /**
     * Возвращает массив с лог сообщениями
     *
     * @var array
     */
    public function getLog () {
        return $this->logs;
    }

    /**
     * Инициализация временных переменных значениями по умолчанию
     */
    private function init () {
        $this->zsource_txt = new zsource_txt();
        $this->buffer = '';
        $this->nextPageLevel = 0;
        $this->levels = array();
        /**
         * Подготавливаем список всех старых страниц, чтобы их потом можно было
         * удалить
         */
        $this->oldPages = array();
        if ($this->deleteOld) {
            $this->oldPages = array_reverse($this->zsource_txt->getAllPages(), true);
        }
    }

    /**
     * Обрабатывает часть файла
     *
     * @param string $data часть файла
     * @param boolean $last флаг последней части файла
     */
    private function convert ($data, $last) {
    	$this->buffer .= $data;
    	$matches = array();
    	while (preg_match("%^(##+)%m", $this->buffer, $matches, PREG_OFFSET_CAPTURE)) {
    	   $this->savePage(substr($this->buffer, 0, $matches[0][1]));
           $this->nextPageLevel = strlen($matches[0][0]) - 2;
    	   $this->buffer = '@@title=' . substr($this->buffer, $matches[0][1] + strlen($matches[0][0]));
    	}

    	if ($last) {
    	    $this->savePage($this->buffer);
    	}
    }

    /**
     * Обрабатывает данные страницы и сохраняет их в файле
     *
     * @param string $page данные страницы
     */
    private function savePage ($page) {
        if (empty($page)) {
            return;
        }

        /**
         * Обрабатываем страницу и получаем ее свойства
         */
    	$page = $this->zsource_txt->parse(preg_replace("%\r?\n%", "\n", $page));
    	if (empty($page['sort'])) {
    	    $page['sort'] = (int) ++$this->position;
    	}
    	if (empty($page['title'])) {
    	    $this->log('Страница с пустым заголовком: пропускаем');
    	    return;
    	}
    	if (empty($page['charset'])) {
    	    $page['charset'] = $this->charset;
    	}
    	if (empty($page['filter'])) {
    	    $page['filter'] = $this->filter;
    	}
    	if (empty($page['type'])) {
    	    $page['type'] = 'page';
    	}
    	/**
    	 * Формируем название файла
    	 */
    	$pagename = $page['title'];
    	if (!empty($page['file'])) {
    	    $pagename = $page['file'];
    	    unset($page['file']);
    	}
    	if (strcasecmp($page['charset'], 'utf-8') != 0) {
    	    $pagename = $this->filename(iconv($page['charset'], 'utf-8', $pagename));
    	} else {
    	    $pagename = $this->filename($pagename);
    	}
        if (empty($pagename)) {
            $this->log('Не удалось создать название файла для страницы. Используем "untitled".');
            $pagename = 'untitled';
        }
    	if ($this->nextPageLevel == 0 && ($this->forceIndex || !zengine::zsource()->exists('/index'))) {
    	    $pagename = 'index';
    	    $this->forceIndex = false;
    	}

    	/**
    	 * Формируем путь к файлу для сохранения
    	 */
    	$this->levels[$this->nextPageLevel] = $pagename;
    	$directory = implode('/', array_slice($this->levels, 0, $this->nextPageLevel));
    	assert(strpos($directory, '//') === false);
    	$pagepath = '/' . (empty($directory) ? $pagename : $directory . '/' . $pagename);
        $this->log('Создана страница: ' . $pagepath);

    	/**
    	 * Если страница помечена на удаление, то исключаем её из списка
    	 */
    	if (isset($this->oldPages[$pagepath])) {
    	    unset($this->oldPages[$pagepath]);
    	}

    	/**
    	 * Сохраняем страницу
    	 */
    	zengine::zsource()->save($pagepath, $page);
    }

    /**
     * Записывает сообщение в лог
     *
     * @param string $message
     */
    private function log ($message) {
    	$this->logs[] = $message;
    }

    /**
     * Метод переводит название файла в латинские символы
     *
     * @param string $filename название файла без расширения в кодировке utf-8
     * @return string название файла без расширения латинскими буквами
     */
    private function filename ($filename) {
        $filename = strtolower(zutils::translit($filename));
        $filename = preg_replace('%&.+?;%', '', $filename);
        $filename = str_replace('_', '-', $filename);
        $filename = preg_replace('%[^a-z0-9\s-.]+%', '', $filename);
        $filename = preg_replace('%[+ -]+%', '-', $filename);
        $filename = trim($filename, '-');
        return $filename;
    }

    /**
     * Массив папок по уровням. Используется как временное хранилище пути
     *
     * @var array
     */
    private $levels = array();

    /**
     * Кодировка, используемая по умолчанию
     *
     * @var string
     */
    private $charset = 'utf-8';

    /**
     * Лог
     *
     * @var array
     */
    private $logs = array();

    /**
     * Временная переменная для хранения буфера при разборе страницы
     *
     * @var string
     */
    private $buffer;

    /**
     * Временная переменная, используемая во время парсинга для хранения уровня
     * следующей найденной страницы
     *
     * @var int
     */
    private $nextPageLevel;

    /**
     * Источник данных, в формате которого хранится информация о странице
     *
     * @var zsource_txt
     */
    private $zsource_txt;

    /**
     * Флаг принудительного создания главной страницы
     *
     * @var boolean
     */
    private $forceIndex = false;

    /**
     * Флаг удаления старого контента
     *
     * @var boolean
     */
    private $deleteOld = true;

    /**
     * Массив старых файлов, которые нужно будет удалить
     *
     * @var array
     */
    private $oldPages = array();

    /**
     * Фильтр по умолчанию
     *
     * @var string
     */
    private $filter = 'text2html';

    /**
     * Позиция в меню
     *
     * @var int
     */
    private $position;
}
