<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: defaults.php 502 2008-11-13 09:25:02Z lks $
 */


/**
 * @package    zengine
 * @subpackage widgets
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zwidget_pages extends zwidget
{
    /**
     * Отображает содержимое виджета
     */
    public function displayContent ()
    {
        $this->nofollow = '';
        if (zengine::config()->get('menu.nofollow')) {
            $this->nofollow = ' rel="nofollow"';
        }

        $this->zpage = zengine::ztemplate()->zpage;
        $this->pagepath = $this->zpage->getPagepath();

        echo '<ul>';
        $this->displayItems(zengine::zsource()->getMenu());
        echo '</ul>';
    }

    /**
     * Выводим подпункты меню
     *
     * @param zmenu $items
     */
    private function displayItems ($items)
    {
        foreach ($items as $item) {
            if ($item->nomenuitem) {
                continue;
            }

            $itemClasses = array('page_item');
            // Мы находимся на этой странице
            if ($this->pagepath == $item->uri) {
                $itemClasses[] = 'current_page_item';
            }
            // Эта страницы выше по нашей ветке
            if (strpos($this->pagepath, $item->uri . '/') === 0) {
                $itemClasses[] = 'current_page_ancestor';
                // Эта страницы является родительской для текущей
                if (false === strpos(substr($this->pagepath, strlen($item->uri) + 1), '/')) {
                    $itemClasses[] = 'current_page_parent';
                }
            }

            echo '<li class="' . implode(' ', $itemClasses) . '">';

            if (strpos($this->pagepath, $item->uri . '/') === 0 || strpos($item->uri, dirname($this->pagepath)) !== 0) {
                echo '<a href="' . $item->link . '"' . $this->nofollow . '>' . $item->name . '</a>';
            } else {
                echo '<a href="' . $item->link . '">' . $item->name . '</a>';
            }

            if ($item->hasSubmenu() && $this->showSubmenu($item)) {
                echo '<ul>';
                $this->displayItems($item);
                echo '</ul>';
            }

            echo '</li>';
        }
    }

    /**
     * Метод проверяет, нужно ли выводить подпункты меню данного пункта
     *
     * @param zmenu $item пункт меню
     * @return boolean
     */
    private function showSubmenu ($item) {
        if ($this->getParam('type', 'minimal') == 'minimal') {
            if ($item->uri == $this->pagepath || strpos($this->pagepath, $item->uri . '/') === 0) {
               return true;
            } else {
                return false;
            }
        }
        return true;
    }

    /**
     * Дополнительные атрибуты к пунктам меню
     *
     * @var string
     */
    private $nofollow;

    /**
     * Путь к текущей странице
     *
     * @var string
     */
    private $pagepath;

    /**
     * Опции виджета по умолчанию
     *
     * @var array
     */
    protected $defaults = array(
        'type' => 'minimal'
    );
}

/**
 * @package    zengine
 * @subpackage widgets
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zwidget_tagcloud extends zwidget
{
    /**
     * Отображает содержимое виджета
     */
    public function displayContent () {
        $tags = array();
        $zmenu = zengine::zsource()->getMenu();
        $zmenu->extractTags($tags);
        if (empty($tags)) {
            return;
        }

        // Сортируем тэги
        uasort($tags, array($this, 'tagcmp'));
        if ('DESC' == $this->getParam('order')) {
            $tags = array_reverse($tags, true);
        } elseif ('RAND' == $this->getParam('order')) {
            $keys = array_rand($tags, count($tags));
            foreach ($keys as $key) {
                $temp[$key] = $tags[$key];
            }
            $tags = $temp;
            unset($temp);
        }

        // Ограничиваем количество тэгов
        if ($this->getParam('number') > 0) {
            $tags = array_slice($tags, 0, $this->getParam('number'));
        }

        // Высчитываем шаг для увеличения шрифта тэга
        $counts = array(); {
        foreach ($tags as $key => $tag )
            $counts[$key] = $tag['count'];
        }
        $min_count = min($counts);
        $spread = max($counts) - $min_count;
        if ($spread <= 0) {
            $spread = 1;
        }
        $font_spread = $this->getParam('largest') - $this->getParam('smallest');
        if ($font_spread <= 0) {
            $font_spread = 1;
        }
        $font_step = $font_spread / $spread;

        // Формируем ссылки на тэги
        $a = array();
        foreach ($tags as $link => $tag) {
            $count = $tag['count'];
            $tag_link = zengine::getModule('ztags')->getTagLink($link);
            $tag_id = isset($tag['id']) ? $tag['id'] : $link;
            $tag_name = $tag['name'];
            $a[] = "<a href='$tag_link' class='tag-link-$tag_id' title='" . $count . "' style='font-size: " .
                ($this->getParam('smallest') + (($count - $min_count) * $font_step))
                . $this->getParam('unit') . ";'>$tag_name</a>";
        }

        // Выводим облако тэгов
        switch ($this->getParam('format')) {
        case 'list' :
            echo "<ul class='wp-tag-cloud'>\n\t<li>";
            echo implode("</li>\n\t<li>", $a);
            echo "</li>\n</ul>\n";
            break;
        default :
            echo implode("\n", $a);
            break;
        }
    }

    /**
     * Сортирует тэги между собой
     *
     * @param array $left
     * @param array $right
     * @return int
     */
    public function tagcmp ($left, $right) {
        $cmpkey = $this->getParam('orderby');
        if (!isset($left[$cmpkey]) || !isset($right[$cmpkey])) {
            $cmpkey = 'name';
        }

        $fnCmp = 'strcmp';
        if ($cmpkey == 'name') {
            $fnCmp = 'strnatcasecmp';
        }

        return $fnCmp($left[$cmpkey], $right[$cmpkey]);
    }

    /**
     * Опции виджета по умолчанию
     *
     * @var array
     */
    protected $defaults = array(
        'smallest' => 8,
        'largest' => 22,
        'unit' => 'pt',
        'number' => 45,
        'format' => 'flat',
        'orderby' => 'name',
        'order' => 'ASC'
    );
}
