<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zcache.php 447 2008-10-21 05:21:25Z lks $
 */


/**
 * Класс для работы с кэшем
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zcache
{
    /**
     * Кэш не проверяется на TTL
     */
    const TTL_NEVER_EXPIRES = 0;

    /**
     * Папка для кэша
     *
     * @var string
     */
    private $_cachedir;

    /**
     * Создает пространство для кэша
     *
     * @param string $cachedir папка для кэш файлов
     */
    public function __construct ($cachedir) {
    	if (!is_dir($cachedir)) {
    	    mkdir($cachedir, 0777);
    	}

        $this->_cachedir = realpath($cachedir);

    	if (!file_exists($this->_cachedir . '/drop.php')) {
    	    zutils::file_put_contents($this->_cachedir . '/drop.php', '<?php
foreach (new DirectoryIterator(dirname(__FILE__)) as $file) {
    if ($file->isDot()) {
        continue;
    }
    @unlink($file->getPathname());
}
echo "cache droped";
');
    	}
    }

    /**
     * Сохраняет в кэше информацию под именем $name
     *
     * @param string $name
     * @param mixed $value
     */
    public function put ($name, $value) {
        if (0 == zengine::config()->get('engine.ttl')) {
            return;
        }
        $filename = $this->tempnam($this->_cachedir, 'tmp');
        if (null === $filename) {
            return;
        }
    	zutils::file_put_contents($filename, serialize($value));
    	if (file_exists($this->_cachedir . '/' . $name . '.cache')) {
    	   @unlink($this->_cachedir . '/' . $name . '.cache');
    	}
    	@rename($filename, $this->_cachedir . '/' . $name . '.cache');
    	zutils::chmod($this->_cachedir . '/' . $name . '.cache', 0666);
        zutils::touch($this->_cachedir . '/' . $name . '.cache', time());
    }

    /**
     * Аналогична функции tempnam, за исключением того, что не пробует создать
     * файл во временной директории
     *
     * @param string $dir
     * @param string $prefix
     * @return string
     */
    public function tempnam ($dir, $prefix = '') {
    	if (!is_dir($dir)) {
    	    return null;
    	}

    	$filename = $dir . '/' . $prefix . md5(time());
    	return $filename;
    }

    /**
     * Берет из кэша информацию под именем $name. Если информации нет, то
     * возвращает false
     *
     * @param string $name название переменной
     * @param int $ttl максимальное время жизни кэша в секундах
     * @return mixed
     */
    public function get ($name, $ttl = -1) {
        if (isset($this->_cache[$name])) {
            return $this->_cache[$name];
        }
        if ($ttl == -1) {
            $ttl = zengine::config()->get('engine.ttl');
            if ($ttl == 0) {
                return false;
            }
        }
        $filename = $this->_cachedir . '/' . $name . '.cache';
        if (!file_exists($filename) || ($ttl && filemtime($filename) + $ttl < time())) {
            return false;
        }
    	$this->_cache[$name] = @unserialize(file_get_contents($filename));
    	return $this->_cache[$name];
    }

    /**
     * Возвращает время создания кэша
     *
     * @param string $name название переменной
     * @return int
     */
    public function mtime ($name) {
        $filename = $this->_cachedir . '/' . $name . '.cache';
        if (file_exists($filename)) {
            return filemtime($filename);
        }

        return 0;
    }

    /**
     * Очищает кэш
     */
    public function erase () {
        foreach (new DirectoryIterator($this->_cachedir) as $file) {
            if ($file->isDot() || $file->isDir()) {
                continue;
            }
            @unlink($file->getPathname());
        }
    }

    /**
     * Внутренний массив для уже полученных данных
     *
     * @var array
     */
    private $_cache = array();
}
