<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zengine.php 502 2008-11-13 09:25:02Z lks $
 */

/** zconfig */
require_once 'zengine/classes/zconfig.php';

/** zcache */
require_once 'zengine/classes/zcache.php';

/** zsource */
require_once 'zengine/classes/zsource.php';

/** zpage */
require_once 'zengine/classes/zpage.php';

/** zrouter */
require_once 'zengine/classes/zrouter.php';

/** ztemplate */
require_once 'zengine/classes/ztemplate.php';

/** zutils */
require_once 'zengine/classes/zutils.php';

/** zwidget */
require_once 'zengine/classes/zwidget.php';

/** zsidebar */
require_once 'zengine/classes/zsidebar.php';

/** zsidebarmanager */
require_once 'zengine/classes/zsidebarmanager.php';

/** стандартные виджеты */
require_once 'zengine/classes/widgets/defaults.php';

/**
 * Основной класс публичной части. Отвечает за разбор пришедшего запроса,
 * получения объекта страницы и оформления страницы.
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zengine
{
    /**
     * Номер версии движка
     *
     */
    const VERSION = '1.1.0';

    /**
     * Возвращает версию движка
     *
     * @return string
     */
    static public function getVersion () {
        return self::VERSION;
    }

    /**
     * Запуск движка и показ страницы $uri
     *
     * @param string $uri Запрашиваемая страница
     */
    static public function run ($uri)
    {
        self::init();

        // Настройка правил переадресации
        if (file_exists('zengine/rewrite_rules.php') && is_readable('zengine/rewrite_rules.php')) {
            require_once 'zengine/rewrite_rules.php';
        }

        // Возможность провести настройку движка перед запуском диспетчера
        if (file_exists('zengine/init.php') && is_readable('zengine/init.php')) {
            require_once 'zengine/init.php';
        }

        // Запуск диспетчера
        try {
            self::$_zengine->dispatch($uri);
        } catch (Exception $e) {
            header('HTTP/1.1 500 Internal Server Error');
            echo $e->getMessage();
            // 500
        }
    }

    /**
     * Метод инициализирует движок, но не запускает обработку страницы
     */
    static public function init () {
        if (is_null(self::$_zengine) && !defined('ZENGINE_VERSION')) {
            define('ZENGINE_VERSION', self::getVersion());
        }

        self::$_zengine = new zengine();
        self::$_zengine->configure('config.ini');
        self::$_zengine->determineBaseUri();
        self::$_zengine->_zrouter = new zrouter();

        /**
         * Устанавливаем часовой пояс
         */
        date_default_timezone_set(zengine::config()->get('engine.timezone', 'Europe/Moscow'));

        /**
         * Текущее время, можно менять для тестирования
         */
        if (! defined('ZENGINE_NOW')) {
            define('ZENGINE_NOW', date('Y-m-d H:i:s'));
        }

        /**
         * Очистка кэша, если требуется
         */
        $hash = self::$_zengine->_cache->get('confighash');
        if (false === $hash || $hash != self::$_zengine->getSettingsHash()) {
            self::$_zengine->_cache->erase();
            self::$_zengine->_cache->put('confighash', self::$_zengine->getSettingsHash());
        }
    }

    /**
     * Возвращает объект роутера
     *
     * @return zrouter
     */
    public static function getRouter () {
    	return self::$_zengine->_zrouter;
    }

    /**
     * Возвращает экземпляр запущенного движка
     *
     * @return zengine
     */
    static public function getInstance ()
    {
        return self::$_zengine;
    }

    /**
     * Возвращает базовый uri
     *
     * @return string
     */
    public function getBaseUri ()
    {
        return $this->_baseuri;
    }

    /**
     * Определяет базовый URI для сайта
     *
     */
    public function determineBaseUri () {
        if (defined('ZENGINE_BASEURI')) {
            $this->_baseuri = ZENGINE_BASEURI;
        } else {
            $baseuri = $this->config()->get('url.baseuri', '');

            /**
             * Если url.baseuri не задан в конфигурационном файле, то пробуем
             * его вычислить автоматически
             */
            if (empty($baseuri)) {
                $baseuri = '/';

                /**
                 * Поиск по разнице между запрошенным адресом и тем, что пришел от
                 * браузера
                 */
                if ($_SERVER['REQUEST_URI'] != $this->_uri && substr($_SERVER['REQUEST_URI'], - strlen($this->_uri)) == $this->_uri) {
                    $baseuri = substr($_SERVER['REQUEST_URI'], 0, - strlen($this->_uri)) . '/';
                } else {
                    /**
                     * Поиск по разнице между корневой директорией и предполагаемой
                     * корневой директорией
                     */
                    $proposedDocumentRoot = rtrim(realpath(dirname(__FILE__) . '/../../'), DIRECTORY_SEPARATOR);
                    $realDocumentRoot = rtrim(realpath($_SERVER['DOCUMENT_ROOT']), DIRECTORY_SEPARATOR);
                    if ($proposedDocumentRoot != $realDocumentRoot && substr($proposedDocumentRoot, 0, strlen($realDocumentRoot)) == $realDocumentRoot) {
                        $baseuri = str_replace(DIRECTORY_SEPARATOR, '/', substr($proposedDocumentRoot, strlen($realDocumentRoot))) . '/';
                    }
                }
            } else {
                $baseuri = str_replace('//', '/', '/' . trim($baseuri, '/') . '/');
            }

            $this->_baseuri = $baseuri;
        }
    }

    /**
     * Возвращает хэш-код настроек системы
     *
     * @return string
     */
    public function getSettingsHash () {
    	return md5($this->getBaseUri(false) . $this->config()->getHash() . ZENGINE_VERSION);
    }

    /**
     * Возвращает объект модуля. Если объект не был создан, то он создается
     *
     * @param string $module
     * @return zmodule
     */
    static public function getModule ($module) {
        if (!isset(self::$_modules[$module])) {
            self::$_modules[$module] = null;
            $moduleFile = 'templates/' . zengine::config()->get('template.theme', 'default') . '/modules/' . $module . '/' . $module . '.php';
            if (! file_exists($moduleFile)) {
                $moduleFile = 'zengine/modules/' . $module . '/' . $module . '.php';
            }
            if (file_exists($moduleFile)) {
                include_once $moduleFile;
                $className = 'zmodule_' . $module;
                if (class_exists($className, false)) {
                    $zmodule = new $className();
                    if ($zmodule instanceof zmodule) {
                        self::$_modules[$module] = $zmodule;
                    }
                }
            }
        }
        return self::$_modules[$module];
    }

    /**
     * Возвращает запрошенную в данный момент страницу
     *
     * @return string
     */
    public function getFilepath () {
    	return $this->_filepath;
    }

    /**
     * Возвращает true, еслю включен режим отладки
     *
     * @return boolean
     */
    static public function debugMode () {
    	return self::$_zengine->_debug;
    }

    /**
     * Возвращает объект конфигурации
     *
     * @return zconfig
     */
    static public function config ()
    {
        return self::$_zengine->_config;
    }

    /**
     * Возвращает объект источника данных
     *
     * @return zsource
     */
    static public function zsource ()
    {
        return self::$_zengine->_zsource;
    }

    /**
     * Возвращает объект для рабоыт с кэшем
     *
     * @return zcache
     */
    static public function zcache ()
    {
        return self::$_zengine->_cache;
    }

    /**
     * Возвращает объект шаблона
     *
     * @return ztemplate
     */
    static public function ztemplate ()
    {
        return self::$_zengine->_ztemplate;
    }

    /**
     * Обработка запроса
     *
     * @param string $uri Запрашиваемая страница
     */
    private function dispatch ($uri)
    {
        /**
         * Запоминаем запрошенную страницу
         */
        $this->_uri = $uri;

        /**
         * Находим необходимую нам страницу
         */
        $zpage = $this->_zrouter->route($uri);
        if (zengine::debugMode()) {
            ob_start();
            echo '<pre>';
            echo 'Серверное время: ' . ZENGINE_NOW . "\n";
            if ($zpage) {
                echo 'Время публикации страницы: ' . $zpage->getProperty('publish') . "\n";
                echo 'Название страницы: ' . $zpage->getTitle();
            } else {
                echo 'Страница не найдена';
            }
            echo '</pre>';
        }
        if (!$zpage || $zpage->getProperty('publish') > ZENGINE_NOW) {
            header('HTTP/1.1 404 Not Found');
            require_once 'zengine/classes/zpage/error.php';
            $zpage = zpage_error::getNotFound();
        }

        $this->_ztemplate = new ztemplate();
        echo $this->_ztemplate->drawup($zpage);
    }

    /**
     * Закрытый конструктор
     */
    private function __construct ()
    {
    }

    /**
     * Возвращает номер подстраницы и изменяет значение переменной $uri на
     * новый адрес запроса, по которому находится настоящая страница
     *
     * @param string $uri Без расширения
     * @return int
     */
    public function extractSubpageNumber (&$uri) {
        $pageuripos = 2;
        $pagenumpos = 1;
        /**
         * Подготавливаем шаблон для регулярного выражения
         */
        $pattern = preg_quote($this->_config->get('split.url', '#pageuri#-#pagenum#'), '%');
        if (strpos($pattern, '#pageuri#') < strpos($pattern, '#pagenum#')) {
            $pageuripos = 1;
            $pagenumpos = 2;
        }

        $pattern = '%^' . str_replace(array('#pagenum#', '#pageuri#'), array('(\d+)', '(.+?)'), $pattern) . '$%';

        /**
         * Пробуем сопоставить шаблон с запрошенной страницей
         */
        $subpage = 1;
        if (preg_match($pattern, basename($uri), $matches)) {
            $uri = rtrim(dirname($uri), '/') . '/' . $matches[$pageuripos];
            $subpage = $matches[$pagenumpos];
            if ($subpage == 1) {
                return 0;
            }
        }

        return $subpage;
    }

    /**
     * Метод конфигурирует движок
     *
     * @param string $configfile путь к ini-файлу
     */
    private function configure ($configfile)
    {
        // Инициализация свойств по умолчанию
        $this->_baseuri = null;

        $this->_config = new zconfig($configfile);
        $this->_debug = (bool) $this->_config->get('engine.debug', false);
        /**
         * Устанавливаем значения по умолчанию для часто используемых параметров
         */
        $this->_config->setDefault('engine.charset', 'utf-8');
        $this->_config->setDefault('source.charset', 'utf-8');
        $this->_config->setDefault('source.extension', 'txt');
        $this->_config->setDefault('url.extension', 'html');
        /**
         * Корректируем права доступа
         */
        if (is_string($this->_config->get('directory.chmod'))) {
            $this->_config->set('directory.chmod', octdec($this->_config->get('directory.chmod')));
        } else {
            $this->_config->set('directory.chmod', 0777);
        }
        if (is_string($this->_config->get('file.chmod'))) {
            $this->_config->set('file.chmod', octdec($this->_config->get('file.chmod')));
        } else {
            $this->_config->set('file.chmod', 0666);
        }
        /**
         * Создание экземпляра источника данных
         */
        $source = $this->_config->get('source.engine', 'txt');
        $sourceClass = 'zsource_' . $source;
        if (! class_exists($sourceClass, false)) {
            if (! file_exists('zengine/classes/zsource/' . $source . '.php')) {
                throw new Exception('Invalid zsource');
            }
            require_once 'zengine/classes/zsource/' . $source . '.php';
            if (! class_exists($sourceClass, false)) {
                throw new Exception('Class ' . $sourceClass . ' is not found');
            }
        }
        $this->_zsource = new $sourceClass();
        $this->_cache = new zcache('cache');
    }

    /**
     * Объект с конфигурацией движка
     *
     * @var zconfig
     */
    private $_config;

    /**
     * Экземпляр источника данных о страницах
     *
     * @var zsource
     */
    private $_zsource;

    /**
     * Запрос, который в данный момент обрабатывается
     *
     * @var string
     */
    private $_uri;

    /**
     * Запрос без расширения файла, который в данный момент обрабатывается
     *
     * @var string
     */
    private $_filepath;

    /**
     * Базовый uri
     *
     * @var string
     */
    private $_baseuri;

    /**
     * Экземпляр запущенного движка
     *
     * @var zengine
     */
    static private $_zengine;

    /**
     * Экземпляр класса для работы с кэшем
     *
     * @var zcache
     */
    private $_cache;

    /**
     * Массив объектов модулей
     *
     * @var array
     */
    static private $_modules;

    /**
     * Экземпляр использованного шаблона. Используется только для быстрых функций шаблонизатора
     *
     * @var ztemplate
     */
    private $_ztemplate;

    /**
     * Экземпляр класса роутера
     *
     * @var zrouter
     */
    private $_zrouter;

    /**
     * Флаг режима отладки
     *
     * @var boolean
     */
    private $_debug;
}
