<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zhelper.php 203 2008-08-30 13:08:54Z lks $
 */


/**
 * Класс-помощник для работы с шаблонами. Используется для быстрого вызова
 * помощников из шаблона.
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zhelper {
    /**
     * Магический метод, который проксирует все методы в классы помощников.
     * Весь вывод помощника буферизируется и отдается вызывающему методу.
     *
     * @param string $name
     * @param array $arguments
     * @return string
     */
    public function __call ($name, $arguments) {
        $helperClass = 'zhelper_' . $name;

        /**
         * Создание объекта помощника
         */
        if (! class_exists($helperClass, false)) {
            if (file_exists('templates/' . $this->ztemplate->getTheme() . '/helpers/' . $name . '.php')) {
                include_once 'templates/' . $this->ztemplate->getTheme() . '/helpers/' . $name . '.php';
            } elseif (file_exists('zengine/helpers/' . $name . '.php')) {
                include_once 'zengine/helpers/' . $name . '.php';
            }
            if (! class_exists($helperClass, false)) {
                return $helperClass;
            }
        }
        $helper = new $helperClass();
        foreach ($this->variables as $key => $value) {
            $helper->$key = $value;
        }
        /**
         * Вызов помощника
         */
        ob_start();
        call_user_func_array(array($helper, $name), $arguments);
        $content = ob_get_contents();
        ob_end_clean();
        unset($helper);
        return $content;
    }

    /**
     * Магический метод __get()
     *
     * @param string $name название свойства класса
     * @return mixed
     */
    public function __get ($name) {
    	if (isset($this->variables[$name])) {
    	    return $this->variables[$name];
    	}
    	return null;
    }

    /**
     * Магический метод __set()
     *
     * @param string $name
     * @param mixed $value
     */
    public function __set ($name, $value) {
    	$this->variables[$name] = $value;
    }

    /**
     * Массив с параметрами, которые необходимо передать помощнику
     *
     * @var array
     */
    private $variables = array();
}
