<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zrouter.php 497 2008-11-13 07:54:17Z lks $
 */

/** zroute */
require_once 'zengine/classes/zroute.php';

/**
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zrouter
{
    /**
     * Правила переадресации
     *
     * @var array
     */
    private $_routes = array();

    /**
     * Конструктор класса
     *
     * @param zconfig $routes Правила роутера
     */
    public function __construct ()
    { }

    /**
     * Добавляет маршрут
     *
     * @param string $name
     * @param zroute $route
     * @return zrouter
     */
    public function addRoute ($name, zroute $route) {
    	$this->_routes[$name] = $route;
    	return $this;
    }

    /**
     * Удаляет маршрут с именем <code>$name</code>
     *
     * @param string $name
     * @return zrouter
     */
    public function removeRoute ($name) {
    	if (isset($this->_routes[$name])) {
    	    unset($this->_routes[$name]);
    	}
    	return $this;
    }

    /**
     * Возвращает маршрут с именем <code>$name</code>
     *
     * @param string $name
     * @return zroute
     */
    public function getRoute ($name) {
    	if (isset($this->_routes[$name])) {
    	    return $this->_routes[$name];
    	}

    	return null;
    }

    /**
     * Обрабатывает пришедший запрос и возвращает объект найденной страницы
     *
     * @param string $url
     * @return zpage
     */
    public function route ($url) {
        /**
         * Убираем базовый путь
         *
         * @todo Если это условие не выполняется, то скорее всего должна быть 404 ошибка
         */
        if (substr($url, 0, strlen(zengine::getInstance()->getBaseUri())) == zengine::getInstance()->getBaseUri()) {
            $url = substr($url, strlen(zengine::getInstance()->getBaseUri()) - 1);
        }

        // Полный путь к файлу
        $uri = $url;

        /**
         * Формируем путь к странице
         */
        $isPage = false;
        $pagepath = $uri;
        $ext = zengine::getInstance()->config()->get('url.extension');
        if (!empty($ext) && substr($pagepath, - strlen($ext) - 1) == '.' . $ext) {
            $pagepath = substr($uri, 0, strlen($pagepath) - strlen($ext) - 1);
            $isPage = true;
        }

        /**
         * Если найден модуль, отвечающий за обработку запроса, создаем его и
         * запрашиваем у него страницу
         */
        $arModule = null;
        foreach (array_reverse($this->_routes) as $route) {
        	if (false !== ($arModule = $route->match($url, $pagepath))) {
                $zmodule = zengine::getModule($arModule['module']);
                if ($zmodule) {
                    if (is_array($arModule['params'])) {
                        $zmodule->setParams($arModule['params']);
                    }
                    $zmodule->setParam('isPage', $isPage);
                    $zmodule->setParam('basePagepath', $route->getPagepath());
                    return $zmodule->getPage($pagepath);
                }
        	}
        }

        return null;
    }
}
