<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: ztemplate.php 492 2008-11-12 13:42:53Z lks $
 */

/** ztemplate_exception*/
require_once 'zengine/classes/ztemplate/exception.php';

/** zhelper */
require_once 'zengine/classes/zhelper.php';

/** вспомогательные функции для шаблонов */
require_once 'zengine/classes/ztemplate/functions.php';

/**
 * Класс для работы с шаблонами
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class ztemplate {
    /**
     * Название темы сайта
     *
     * @var string
     */
    private $theme;

    /**
     * Название шаблона, в который оформляется страницы
     *
     * @var string
     */
    private $template;

    /**
     * Экземпляр класса оформляемой страницы
     *
     * @var zpage
     */
    public $zpage;

    /**
     * Кодировка генерируемой html-страницы
     *
     * @var string
     */
    public $charset;

    /**
     * Конструктор.
     */
    public function __construct () {
        /**
         * Создание и инициализация помощника
         */
        $this->helper = new zhelper();
        $this->helper->ztemplate = $this;
    }

    /**
     * Оформляет переданную страницу в шаблон и возвращает оформленную страницу
     *
     * @param zpage $zpage
     * @return string
     * @throws ztemplate_exception
     */
    public function drawup (zpage $zpage) {
        $this->zpage = $zpage;
        $this->helper->zpage = $zpage;
        /**
         * Определяем тему и шаблон
         */
        $theme = zengine::config()->get('template.theme', 'default');
        $template = $zpage->getTemplate();
        if (empty($template)) {
            $template = zengine::config()->get('template.default', 'template');
        }

        /**
         * Определяемся с кодировкой html-страницы
         */
        $this->charset = zengine::config()->get('engine.charset');
        header('Content-Type: text/html; charset=' . $this->charset, true);

        /**
         * Подключение пользовательских функций, если они есть
         */
        if (file_exists('templates/' . $theme . '/functions.php')) {
            include_once 'templates/' . $theme . '/functions.php';
        }

        /**
         * Подключаем шаблон
         */
        try {
            $content = $this->applyTemplate($theme, $template);
        } catch (ztemplate_exception $e) {
            if ($theme == 'default') {
                throw $e;
            }
            $content = $this->applyTemplate('default', $template);
        }
        $content = $this->replaceSnippets($content);
        echo $content;
    }

    /**
     * Возвращает название текущей темы
     *
     * @return string
     */
    public function getTheme () {
        if (empty($this->theme)) {
            throw new ztemplate_exception('No theme used');
        }
    	return $this->theme;
    }

    /**
     * Возвращает название текущего шаблона
     *
     * @return string
     */
    public function getTemplate () {
        if (empty($this->template)) {
            throw new ztemplate_exception('No template used');
        }
    	return $this->template;
    }

    /**
     * Загружает дополнительный шаблон и возвращает его
     *
     * @param string $filename
     * @return string
     */
    public function loadTpl ($filename) {
    	//if (file_exists('templates/' . $this->theme ))
    }

    /**
     * Замена системных сниппетов шаблона
     *
     * @param string $content
     * @return string
     */
    private function replaceSnippets ($content) {
    	$snippets = array(
    	   '~/' => zengine::getInstance()->getBaseUri(),
           '~zfull/' => 'http://'. $_SERVER['HTTP_HOST'] . zengine::getInstance()->getBaseUri(),
    	   '~ztpl/' => zengine::getInstance()->getBaseUri() . 'templates/' . $this->theme . '/',
    	);
    	return str_replace(array_keys($snippets), array_values($snippets), $content);
    }

    /**
     * Подключает шаблон
     *
     * @param string $theme
     * @param string $template
     * @return string
     */
    private function applyTemplate ($theme, $template) {
        $templateFile = $template . '.' . zengine::config()->get('template.extension', 'php');
        if (!file_exists('templates/' . $theme . '/' . $templateFile) || !is_readable('templates/' . $theme . '/' . $templateFile)) {
            throw new ztemplate_exception("Unable to load template '{$theme}:{$template}'");
    	}

    	$this->theme = $theme;
    	$this->template = $template;

    	$templateVersion = $this->getTemplateVersion($theme, $template);
    	$this->checkBackwardCompatibility($templateVersion);

    	ob_start();
        include('templates/' . $this->theme . '/' . $templateFile);
        $content = ob_get_contents();
        ob_end_clean();
        return $content;
    }

    /**
     * Возвращает номер версии шаблона
     *
     * @param string $theme
     * @param string $template
     * @return string
     */
    private function getTemplateVersion ($theme, $template) {
    	if (!file_exists('templates/' . $theme . '/.version')) {
    	    // Номер версии системы, где была добавлена проверка на версию
    	    return '1.0.2';
    	}
    	return trim(file_get_contents('templates/' . $theme . '/.version'));
    }

    /**
     * Проверяет совместимость шаблона и в случае несовместимости, выводит
     * сообщение об ошибке
     *
     * @param string $version
     * @throws Exception
     */
    private function checkBackwardCompatibility ($version) {
        if (version_compare(ZENGINE_VERSION, $version, '<')) {
            throw new Exception('Для использования данного шаблона необходимо обновить систему до версии не ниже ' . $version);
        }
    }

    /**
     * Возвращает xhtml-фрагмент для вставки в блок <head>
     *
     * @return string
     */
    public function head () {
        $htmls[] = '<title>' . $this->zpage->getTitle() . '</title>';
        $htmls[] = $this->headMeta('keywords', $this->zpage->getKeywords());
        $htmls[] = $this->headMeta('description', $this->zpage->getDescription());
        $htmls[] = '<meta http-equiv="Content-Type" content="text/html; charset=' . $this->charset . '" />';
    	return implode($htmls) . "\n";
    }

    /**
     * Формирует тэг <meta>
     *
     * @param string $name название
     * @param string $content значение
     * @return string
     */
    private function headMeta ($name, $content) {
    	if (!empty($content)) {
    	    return '<meta name="' . htmlspecialchars($name) . '" content="' . htmlspecialchars($content) . '" />';
    	}
    	return '';
    }

    /**
     * Все вызовы функций, которые не существуют, перенпаравляются на помощник
     *
     * @param string $name название функции
     * @param array $arguments массив с параметрами
     * @return mixed
     */
    public function __call ($name, $arguments) {
        return call_user_func_array(array($this->helper, $name), $arguments);
    }

    /**
     * Экземпляр объекта помощника
     *
     * @param zhelper
     */
    public $helper;
}
