<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zutils.php 495 2008-11-12 14:38:09Z lks $
 */


/**
 * Класс инструментов
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zutils {
    /**
     * Функция транслитерации из UTF-8 в ASCII. Перекодируются только русские символы
     * согласно "системе" Yellow Pages (Traveller's Yellow Pages Transliteration) +
     * мягкий и твердный знаки согласно ISO-9, увы
     *
     * @link http://www.langust.ru/etc/translit.shtml О разных системах транслитерации русских символов
     * @param string $string строка в кодировке UTF-8 (перекодируются только русские буквы)
     * @return string строка в кодировке ASCII
     */
    static public function translit ($string) {
        static $arRus = array("/\xd1\x91/u","/\xd0\xb9/u","/\xd1\x86/u","/\xd1\x83/u","/\xd0\xba/u","/\xd0\xb5/u","/\xd0\xbd/u","/\xd0\xb3/u","/\xd1\x88/u","/\xd1\x89/u","/\xd0\xb7/u","/\xd1\x85/u","/\xd1\x8a/u","/\xd1\x84/u","/\xd1\x8b/u","/\xd0\xb2/u","/\xd0\xb0/u","/\xd0\xbf/u","/\xd1\x80/u","/\xd0\xbe/u","/\xd0\xbb/u","/\xd0\xb4/u","/\xd0\xb6/u","/\xd1\x8d/u","/\xd1\x8f/u","/\xd1\x87/u","/\xd1\x81/u","/\xd0\xbc/u","/\xd0\xb8/u","/\xd1\x82/u","/\xd1\x8c/u","/\xd0\xb1/u","/\xd1\x8e/u","/\xd0\x81/u","/\xd0\x99/u","/\xd0\xa6/u","/\xd0\xa3/u","/\xd0\x9a/u","/\xd0\x95/u","/\xd0\x9d/u","/\xd0\x93/u","/\xd0\xa8/u","/\xd0\xa9/u","/\xd0\x97/u","/\xd0\xa5/u","/\xd0\xaa/u","/\xd0\xa4/u","/\xd0\xab/u","/\xd0\x92/u","/\xd0\x90/u","/\xd0\x9f/u","/\xd0\xa0/u","/\xd0\x9e/u","/\xd0\x9b/u","/\xd0\x94/u","/\xd0\x96/u","/\xd0\xad/u","/\xd0\xaf/u","/\xd0\xa7/u","/\xd0\xa1/u","/\xd0\x9c/u","/\xd0\x98/u","/\xd0\xa2/u","/\xd0\xac/u","/\xd0\x91/u","/\xd0\xae/u");
        static $arTranslit = array('e','y','ts','u','k','e','n','g','sh','shch','z','kh','"','f','y','v','a','p','r','o','l','d','zh','e','ya','ch','s','m','i','t','\'','b','yu','E','Y','TS','U','K','E','N','G','SH','SHCH','Z','KH','"','F','Y','V','A','P','R','O','L','D','ZH','E','YA','CH','S','M','I','T','\'','B','YU');

        return preg_replace($arRus, $arTranslit, $string);
    }

    /**
     * Прокси для функции file_put_contents. При включенном режиме debug выводит
     * сообщения об ошибках и возможных их причинах
     *
     * @param string $filename
     * @param mixed $data
     * @return int
     */
    static public function file_put_contents ($filename, $data) {
        if (zengine::debugMode() && file_exists($filename) && !is_writable($filename)) {
            echo '<pre>ОШИБКА: Нет прав на перезапись файла ' . $filename . '. Права доступа на файл: ' . decoct(@fileperms($filename)) . '</pre>';
        }
    	return @file_put_contents($filename, $data);
    }

    /**
     * Прокси для функции chmod.
     *
     * @param string $filename
     * @param int $mode
     * @return boolean
     */
    static public function chmod ($filename, $mode) {
        if (!is_writable($filename)) {
            if (zengine::debugMode()) {
                echo '<pre>ОШИбКА: Изменение прав доступа для файла ' . $filename . ' запрещено. Права доступа на файл: ' . decoct(@fileperms($filename)) . '</pre>';
            }
            return false;
        }
        return @chmod($filename, $mode);
    }

    /**
     * Прокси для функции touch.
     *
     * @param string $filename
     * @param int $time
     * @return boolean
     */
    static public function touch ($filename, $time) {
        if (!is_writable($filename)) {
            return false;
        }
    	return @touch($filename, $time);
    }

}