<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zwidget.php 497 2008-11-13 07:54:17Z lks $
 */


/**
 * Абстрактный класс для виджетов
 *
 * @package    zengine
 * @subpackage Widgets
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
/*abstract*/ class zwidget
{
    /**
     * Инициализация виджета. Метод выполняется после создания виджета
     *
     */
    public function init ()
    { }

    /**
     * Создает виджет
     *
     * @param string $name Идентификатор виджета
     * @param string $title Заголовок виджета, который выводится над виджетом
     */
    public function __construct ($name, array $params = array(), $classname = '') {
    	$this->id = $name;
    	$this->classname = empty($classname) ? 'widget_' . $name : $classname;
    	$this->params = array_merge($this->params, $this->defaults, $params);
    	$this->init();
    }

    /**
     * Выводит обёртку виджета и вызывает вывод содержимого виджета
     *
     * Ключи массива параметров:
     *      before_widget - Контент, который будет добавлен перед виджетом
     *      after_widget - Контент, который будет добавлен после виджета
     *      before_title - Контент, который будет добавлен перед заголовком виджета
     *      after_title - Контент, который будет добавлен после заголовка виджета
     *
     * @uses zwidget::displayContent() для вывода содержимого виджета
     */
    public function display (array $params) {
        $widget = array_merge($this->params, $params);
        $widget['before_widget'] = sprintf($widget['before_widget'], $this->getId(), $this->getClassname());

        echo $widget['before_widget'] . $widget['before_title'] . $this->getParam('title', 'Widget') . $widget['after_title'];
        echo $this->displayContent();
        echo $widget['after_widget'];
    }

    /**
     * Выводит внутреннее содержимое виджета
     *
     */
    public function displayContent () {
        if (isset($this->params['content'])) {
            echo $this->params['content'];
        } elseif (isset($this->params['callback']) && is_callable($this->params['callback'])) {
            $args = array();
            if (isset($this->params['callback_args']) && is_array($this->params['callback_args'])) {
                $args = $this->params['callback_args'];
            }
            echo call_user_func_array($this->params['callback'], $args);
        }
    }

    /**
     * Массив настроек виджета
     *
     * @var array
     */
    private $params = array(
        'before_widget' => '<li id="%1$s" class="widget %2$s">',
        'after_widget' => "</li>\n",
        'before_title' => '<h2 class="widgettitle">',
        'after_title' => "</h2>\n",
    );

    /**
     * Массив настроек виджета по умолчанию
     *
     * @var array
     */
    protected $defaults = array();

    /**
     * Идентификатор виджета
     *
     * @var string
     */
    private $id;

    /**
     * Возвращает идентификатор виджета
     *
     * @return string
     */
    public function getId () {
    	return $this->id;
    }

    /**
     * Название класса для блока виджета
     *
     * @var string
     */
    private $classname;

    /**
     * Возвращает название класса для виджета
     *
     * @return string
     */
    public function getClassname () {
    	return $this->classname;
    }

    /**
     * Возвращает значение параметра <code>$key</code>
     *
     * @param string $key
     * @param string $default
     * @return string
     */
    public function getParam ($key, $default = null) {
    	if (isset($this->params[$key])) {
    	    return $this->params[$key];
    	}

    	return $default;
    }

}
