<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zsitemapxml.php 497 2008-11-13 07:54:17Z lks $
 */


/**
 * Класс для создания карты sitemap.xml
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @see        http://sitemaps.org/protocol.php
 */
class zmodule_zsitemapxml extends zmodule
{
    /**
     * Создает карту сайта, основываясь на меню, и возвращает ее
     *
     * @return string
     */
    public function getPage ($uri)
    {
        header('Content-Type: application/xml; charset=utf-8');

        $xml = zengine::zcache()->get('sitemapxml');
        if (false === $xml) {
            if (class_exists('XMLWriter')) {
                $xml = $this->_xmlwriterCreate();
            } else {
                $xml = $this->_plainCreate();
            }
            zengine::zcache()->put('sitemapxml', $xml);
        }

        echo $xml;
        exit;
    }

    /**
     * Создает карту сайта используя XMLWriter, основываясь на меню, и возвращает ее.
     *
     * @return string
     */
    private function _xmlwriterCreate ()
    {
        $writer = new XMLWriter();
        $writer->openMemory();
        $writer->startDocument('1.0', 'utf-8');
        $writer->startElement('urlset');
        $writer->writeAttribute('xmlns', 'http://www.sitemaps.org/schemas/sitemap/0.9');

        /**
         * Перебираем все пункты меню и выводим, если дата публикации уже
         * наступила
         */
        $zmenu = new RecursiveIteratorIterator(zengine::zsource()->getMenu(), RecursiveIteratorIterator::SELF_FIRST);
        foreach ($zmenu as $item) {
            /* @var $item zmenu */
            if ($item->publish > ZENGINE_NOW) {
                continue;
            }
            $writer->startElement('url');
            $link = 'http://' . $_SERVER['HTTP_HOST'] . $item->link;
            $writer->writeElement('loc', $link);
            $writer->writeElement('lastmod', date('Y-m-d\TH:i:s+00:00', strtotime($item->publish)));
            $writer->endElement();
        }

        $writer->endElement();
        $writer->endDocument();
        return $writer->outputMemory();
    }

    /**
     * Создает карту сайта без XMLWriter, основываясь на меню, и возвращает ее.
     *
     * @return string
     */
    private function _plainCreate ()
    {
        ob_start();
        echo '<?xml version="1.0" encoding="utf-8"?>
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';

        /**
         * Перебираем все пункты меню и выводим, если дата публикации уже
         * наступила
         */
        $zmenu = new RecursiveIteratorIterator(zengine::zsource()->getMenu(), RecursiveIteratorIterator::SELF_FIRST);
        foreach ($zmenu as $item) {
            /* @var $item zmenu */
            if ($item->publish > ZENGINE_NOW) {
                continue;
            }
            echo '<url>';
            $link = 'http://' . $_SERVER['HTTP_HOST'] . $item->link;
            echo '<loc>' . $link . '</loc>';
            echo '<lastmod>' . date('Y-m-d\TH:i:s+00:00', strtotime($item->publish)) . '</lastmod>';
            echo '</url>';
        }

        echo '</urlset>';
        $sitemapxml = ob_get_contents();
        ob_end_clean();
        return $sitemapxml;
    }
}
