<?php
/**
 * Zebrum Lite
 *
 * ЛИЦЕНЗИЯ
 *
 * Этот исходный файл является предметом лицензии, которая включена в данный
 * пакет в файле LICENSE.txt.
 * Она так же доступна через всемирную сеть Интернет по этому URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: images.php 491 2008-11-11 14:13:37Z lks $
 */

/** ztransform */
require_once 'zengine/classes/ztransform.php';

/** zpage */
require_once 'zengine/classes/zpage.php';

/**
 * Расстановка изображений в теле страницы
 *
 * @package    zengine
 * @subpackage ztransform
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class ztransform_images implements ztransform {
    /**
     * Инициализация экземпляра класса
     *
     */
    public function __construct () {
        //$tag = zengine::ztemplate()->loadTpl('transform_images.tpl');
	    if (!empty($tag)) {
	        $this->_tag = $tag;
	    }
    }

    /**
     * Делает необходимые изменения страницы.
     *
     * @param zpage $zpage Объект страницы
     */
    public function transform (zpage $zpage) {
        $this->_images = $this->getImages($zpage);
        $this->_cursor = 0;

        $content = $zpage->getProperty('body');
        $content = preg_replace_callback("%<img(?::([^\s>]+?))?(\s[^>]*)?/?>%i", array($this, 'insertImage'), $content);
        // Если не было сделано ни одной замены, то добавляем первое изображение в начало страницы
        if (!$this->_cursor && $this->_images) {
            $content = $this->insertImage() . $content;
        }
        $zpage->setProperty('body', $content);
    }

    /**
     * Делает вставку изображения
     *
     * @param array $match
     * @return string
     */
    private function insertImage ($matches = array()) {
        // Получаем все перекрывающие свойства изображения для замены
        $attributes = array();
        if (!empty($matches[2])) {
            $attributes = $this->parseAttributes($matches[2]);
            // Если указан источник изображения, то возвращаем тэг как есть
            if (isset($attributes['#src'])) {
                if (substr($attributes['#src'], 0, 2) == './') {
                    // Небольшой патч для контента из симпла
                    return str_replace($attributes['#src'], substr($attributes['#src'], 1), $matches[0]);
                } else {
                    return $matches[0];
                }
            }
        }

        // Определяем изображение
        $image = array();
        if (!empty($matches[1])) {
            $image = $this->getImageArray($matches[1]);
        } else {
            if (isset($this->_images[$this->_cursor])) {
                $image = $this->_images[$this->_cursor];
            }

            // Увеличиваем внутренний указатель
            ++$this->_cursor;
        }

        // Если не удалось ничего узнать про изображение, то возвращаем пустоту
        if (empty($image)) {
            return '';
        }

        // Перекрываем значения по умолчанию значениями, указанными в тэге
        $image['#align'] = 'left';
        $image['#alt'] = '';
        if (!empty($attributes)) {
            $image = array_merge($image, $attributes);
        }

        // Делаем замену
    	$result = str_replace(array_keys($image), array_values($image), $this->_tag);

    	// Удаляем неиспользованные свойства, кроме альта, и возвращаем результат
        return preg_replace("%\s+(?!alt)([a-z]+)\s*=\s*\"#[a-z]+\"%i", '', $result);
    }

    /**
     * Разбирает строку и возвращает ассоциативный массив атрибутов
     *
     * @param string $string
     * @return array
     */
    private function parseAttributes ($string) {
    	$attributes = array();
    	if (preg_match_all("%([a-z]+)\s*=\s*(?:\"([^\"]+)\"|([^\s]+))%Ui", $string, $matches, PREG_PATTERN_ORDER)) {
    	    foreach ($matches[1] as $key => $name) {
    	        $value = $matches[2][$key];
    	        if (empty($value)) {
    	            $value = $matches[3][$key];
    	        }
    	        $attributes['#' . strtolower($name)] = $value;
    	    }
    	}
    	return $attributes;
    }

    /**
     * Возвращает массив изображений, которые нужно использовать в теле
     * страницы. Каждый элемент массива это массив с информацией об изображении
     *
     * [uri] => '~/images/'
     * [path]
     *
     * @param zpage $zpage
     * @return array
     */
    private function getImages (zpage $zpage) {
    	$images = array();
    	foreach (explode(',', $zpage->getProperty('images')) as $name) {
    	    $name = trim($name);
    	    if (!empty($name)) {
    	       $images[] = $this->getImageArray($name);
    	    }
    	}
    	return $images;
    }

    /**
     * Возвращает массив информации об изображении по его именю
     *
     * @param string $name
     * @return array
     */
    private function getImageArray ($name) {
        $image = array();
        if ($name[0] != '~') {
            $filename = '/images/' . $name;
        } else {
            $filename = substr($name, 1);
        }
        $filepath = ZENGINE_HOMEDIR . $filename;
        if (file_exists($filepath)) {
            $stat = getimagesize($filepath);
            if ($stat) {
                $image['#src'] = '~' . $filename;
                $image['#width'] = $stat[0];
                $image['#height'] = $stat[1];
                $image['#dimensions'] = $stat[3];
            }
        }
        return $image;
    }


    /**
     * Массив с изображениями, участвующими на странице
     *
     * @var array
     */
    private $_images;

    /**
     * Номер текущего вставляемого изображения
     *
     * @var int
     */
    private $_cursor = 0;

    /**
     * Тэг изображения с маркерами для замены
     *
     * @var string
     */
    private $_tag = '<img src="#src" width="#width" height="#height" align="#align" title="#title" alt="#alt" />';
}
